'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _user = require('./user');

var _user2 = _interopRequireDefault(_user);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = {
    irc: (0, _debug2.default)('irc'),
    telegram: (0, _debug2.default)('telegram'),
    bridge: (0, _debug2.default)('bridge')
};

var Bridge = function () {
    function Bridge(options) {
        _classCallCheck(this, Bridge);

        this.config = options.config;
        this.ircConnections = {};
        this.registeredCount = 0;
        this.usersCount = 0;
        this.masterUserClient = false;
        this.irc2tgUserMapping = {};
        this.ircOwnUsernames = [];

        this.tgBot = options.tgBot;
        this.ircLib = options.ircLib;

        this.setupMasterIRCUser();
        this.bindEvents();
    }

    _createClass(Bridge, [{
        key: 'setupMasterIRCUser',
        value: function setupMasterIRCUser() {
            var _this = this;

            this.masterUserClient = new this.ircLib.Client(this.config.irc.server, this.config.irc.master_nick, {
                port: this.config.irc.port,
                secure: this.config.irc.ssl,
                autoConnect: true,
                channels: [this.config.irc.channel]
            });

            this.masterUserClient.addListener('message', function (from, to, message) {
                debug.irc(_this.irc2tgUserMapping);
                debug.irc(from + ' => ' + to + ': ' + message);

                debug.irc(_this.ircOwnUsernames);
                debug.irc(from);

                if (to === _this.config.irc.channel && _this.ircOwnUsernames.indexOf(from) === -1) {
                    var userNames = Object.keys(_this.irc2tgUserMapping);
                    for (var i = 0; i < userNames.length; i++) {
                        if (message.match(userNames[i])) {
                            message = message.replace(userNames[i], (_this.irc2tgUserMapping[userNames[i]].username ? '@' : '') + ('' + _this.irc2tgUserMapping[userNames[i]].telegram_name));
                        }
                    }

                    _this.tgBot.sendMessage(_this.config.telegram.group_id, '[IRC/' + from + '] ' + message);
                }
            });

            this.masterUserClient.addListener('join', function (channel, nick, message) {
                debug.irc(nick + ' JOIN ' + channel);

                if (_this.ircOwnUsernames.indexOf(nick) === -1) {
                    _this.tgBot.sendMessage(_this.config.telegram.group_id, '[IRC/' + nick + ' * joined to the channel *]');
                }
            });

            this.masterUserClient.addListener('part', function (channel, nick, message) {
                debug.irc(nick + ' PART ' + channel);

                if (_this.ircOwnUsernames.indexOf(nick) === -1) {
                    _this.tgBot.sendMessage(_this.config.telegram.group_id, '[IRC/' + nick + ' * left the channel *]');
                }
            });

            this.masterUserClient.addListener('registered', function (data) {
                var nick = data.args[0];
                _this.config.irc.master_nick = nick;
            });

            /* istanbul ignore next */
            this.masterUserClient.addListener('error', function (message) {
                debug.irc('error: ', message);
            });

            this.ircOwnUsernames = [this.config.irc.master_nick];
        }
    }, {
        key: 'addUser',
        value: function addUser(userData) {
            var _this2 = this;

            if (this.config.telegram.ignore_users && this.config.telegram.ignore_users.indexOf(userData.id) > -1) {
                return false;
            }

            var user = new _user2.default(userData);
            this.usersCount++;

            debug.bridge('-- ADD USER --');
            debug.bridge(user);
            debug.bridge('-- ADD USER --');

            var userNick = this.config.irc.nick_prefix ? this.config.irc.nick_prefix : '';
            userNick += user.irc_nick + (this.config.irc.nick_suffix ? this.config.irc.nick_suffix : '');

            this.ircConnections[userData.id] = new this.ircLib.Client(this.config.irc.server, userNick, {
                port: this.config.irc.port,
                secure: this.config.irc.ssl,
                autoConnect: false,
                channels: [this.config.irc.channel]
            });

            this.ircConnections[userData.id].addListener('registered', function (user) {
                return function (data) {
                    var nick = data.args[0];
                    user.real_irc_nick = nick;
                    _this2.irc2tgUserMapping[nick] = user;

                    _this2.ircOwnUsernames = [_this2.config.irc.master_nick].concat(Object.keys(_this2.irc2tgUserMapping));

                    if (++_this2.registeredCount === _this2.usersCount) {
                        _this2.updateUsersDb();
                    }
                };
            }(user));

            /* istanbul ignore next */
            this.ircConnections[userData.id].addListener('error', function (message) {
                debug.irc('error: ', message);
            });

            this.ircConnections[userData.id].connect();
        }
    }, {
        key: 'removeUser',
        value: function removeUser(userId) {
            this.ircConnections[userId].disconnect();
            delete this.ircConnections[userId];
        }
    }, {
        key: 'bindEvents',
        value: function bindEvents() {
            var _this3 = this;

            this.tgBot.on('message', function (msg) {
                var chatId = msg.chat.id;

                if (chatId != _this3.config.telegram.group_id) {
                    return false;
                }

                debug.telegram(msg);

                if (!_this3.ircConnections[msg.from.id]) {
                    _this3.addUser(msg.from);
                }

                if (_this3.ircConnections[msg.from.id]) {
                    _this3.ircConnections[msg.from.id].say(_this3.config.irc.channel, msg.text);
                }
            });

            this.tgBot.on('new_chat_participant', function (msg) {
                debug.telegram(msg);

                var chatId = msg.chat.id;

                if (chatId != _this3.config.telegram.group_id) {
                    return;
                }

                if (!_this3.ircConnections[msg.new_chat_participant.id]) {
                    _this3.addUser(msg.new_chat_participant);

                    _this3.updateUsersDb();
                }
            });

            this.tgBot.on('left_chat_participant', function (msg) {
                debug.telegram(msg);

                var chatId = msg.chat.id;

                if (chatId != _this3.config.telegram.group_id) {
                    return;
                }

                if (_this3.ircConnections[msg.left_chat_participant.id]) {
                    _this3.removeUser(msg.left_chat_participant.id);
                }
            });
        }
    }, {
        key: 'updateUsersDb',
        value: function updateUsersDb() {
            var _this4 = this;

            var result = Object.keys(this.irc2tgUserMapping).map(function (userKey) {
                return {
                    id: _this4.irc2tgUserMapping[userKey].id,
                    username: _this4.irc2tgUserMapping[userKey].username,
                    first_name: _this4.irc2tgUserMapping[userKey].first_name,
                    last_name: _this4.irc2tgUserMapping[userKey].last_name
                };
            });

            _fs2.default.writeFile(this.config.users_db, JSON.stringify(result, null, 2), function (wrErr) {
                debug.bridge(wrErr);
            });
        }
    }]);

    return Bridge;
}();

exports.default = Bridge;