'use strict';
/* jshint esversion: 6, node: true */

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _nodeTelegramBotApi = require('node-telegram-bot-api');

var _nodeTelegramBotApi2 = _interopRequireDefault(_nodeTelegramBotApi);

var _irc = require('irc');

var _irc2 = _interopRequireDefault(_irc);

var _user = require('./user');

var _user2 = _interopRequireDefault(_user);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _DEBUG_ = process.env.DEBUG || false;

var Bridge = function () {
  function Bridge(options) {
    _classCallCheck(this, Bridge);

    this.options = options;
    this.ircConnections = {};
    this.registeredCount = 0;
    this.usersCount = 0;
    this.irc2tgUserMapping = {};
    this.ircOwnUsernames = [];

    this.tgBot = new _nodeTelegramBotApi2.default(options.telegram.token, { polling: true });

    this.setupMasterIRCUser();
    this.bindEvents();
  }

  _createClass(Bridge, [{
    key: 'setupMasterIRCUser',
    value: function setupMasterIRCUser() {
      var _this = this;

      var masterUserClient = new _irc2.default.Client(this.options.irc.server, this.options.irc.master_nick, {
        port: this.options.irc.port,
        secure: this.options.irc.ssl,
        autoConnect: true,
        channels: [this.options.irc.channel]
      });

      masterUserClient.addListener('message', function (masterUserClient) {
        return function (from, to, message) {
          if (_DEBUG_) {
            console.log(_this.irc2tgUserMapping);
            console.log(from + ' => ' + to + ': ' + message);

            console.log(_this.ircOwnUsernames);
            console.log(from);
          }

          if (to === _this.options.irc.channel && _this.ircOwnUsernames.indexOf(from) === -1) {
            var userNames = Object.keys(_this.irc2tgUserMapping);
            for (var i = 0; i < userNames.length; i++) {
              if (message.match(userNames[i])) {
                message = message.replace(userNames[i], (_this.irc2tgUserMapping[userNames[i]].username ? '@' : '') + ('' + _this.irc2tgUserMapping[userNames[i]].telegram_name));
              }
            }

            _this.tgBot.sendMessage(_this.options.telegram.group_id, '[IRC/' + from + '] ' + message);
          }
        };
      }(masterUserClient));

      masterUserClient.addListener('registered', function (data) {
        var nick = data.args[0];
        _this.options.irc.master_nick = nick;
      });

      masterUserClient.addListener('error', function (message) {
        if (_DEBUG_) {
          console.log('error: ', message);
        }
      });
    }
  }, {
    key: 'addUser',
    value: function addUser(userData) {
      var _this2 = this;

      var user = new _user2.default(userData);
      this.usersCount++;

      if (_DEBUG_) {
        console.log('-- ADD USER --');
        console.log(user);
        console.log('-- ADD USER --');
      }

      var thisUserClient = new _irc2.default.Client(this.options.irc.server, user.irc_nick, {
        port: this.options.irc.port,
        secure: this.options.irc.ssl,
        autoConnect: false,
        channels: [this.options.irc.channel]
      });

      thisUserClient.addListener('registered', function (data) {
        return function (data) {
          var nick = data.args[0];
          user.real_irc_nick = nick;
          _this2.irc2tgUserMapping[nick] = user;

          _this2.ircOwnUsernames = [_this2.options.irc.master_nick].concat(Object.keys(_this2.irc2tgUserMapping));

          if (++_this2.registeredCount === _this2.usersCount) {
            _this2.updateUsersDb();
          }
        };
      }(user));

      thisUserClient.addListener('error', function (message) {
        if (_DEBUG_) {
          console.log('error: ', message);
        }
      });

      this.ircConnections[userData.id] = thisUserClient;
    }
  }, {
    key: 'removeUser',
    value: function removeUser(userId) {
      this.ircConnections[userId].disconnect();
      delete this.ircConnections[userId];
    }
  }, {
    key: 'start',
    value: function start() {
      var _this3 = this;

      Object.keys(this.ircConnections).map(function (userId) {
        _this3.ircConnections[userId].connect();
      });
    }
  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      var _this4 = this;

      this.tgBot.on('message', function (msg) {
        var chatId = msg.chat.id;
        if (_DEBUG_) {
          console.log(msg);
        }

        if (!_this4.ircConnections[msg.from.id]) {
          _this4.addUser(new _user2.default(msg.from));
        }

        if (_this4.ircConnections[msg.from.id]) {
          _this4.ircConnections[msg.from.id].say(_this4.options.irc.channel, msg.text);
        }
      });

      this.tgBot.on('new_chat_participant', function (msg) {
        if (_DEBUG_) {
          console.log(msg);
        }

        if (!_this4.ircConnections[msg.new_chat_participant.id]) {
          _this4.addUser(new _user2.default(msg.new_chat_participant));

          _this4.updateUsersDb();
        }
      });

      this.tgBot.on('left_chat_participant', function (msg) {
        if (_DEBUG_) {
          console.log(msg);
        }

        if (_this4.ircConnections[msg.left_chat_participant.id]) {
          _this4.removeUser(msg.left_chat_participant.id);
        }
      });
    }
  }, {
    key: 'updateUsersDb',
    value: function updateUsersDb() {
      var _this5 = this;

      var result = Object.keys(this.irc2tgUserMapping).map(function (userKey) {
        return {
          id: _this5.irc2tgUserMapping[userKey].id,
          username: _this5.irc2tgUserMapping[userKey].username,
          first_name: _this5.irc2tgUserMapping[userKey].first_name,
          last_name: _this5.irc2tgUserMapping[userKey].last_name
        };
      });

      _fs2.default.writeFile(this.options.users_db, JSON.stringify(result, null, 2), function (wrErr) {
        if (_DEBUG_) {
          console.log(wrErr);
        }
      });
    }
  }]);

  return Bridge;
}();

exports.default = Bridge;